@@ -2,7 +2,7 @@
 //! [installer][`uv_installer`] and [build][`uv_build`] through [`BuildDispatch`]
 //! implementing [`BuildContext`].
 
-use std::ffi::OsString;
+use std::ffi::{OsStr, OsString};
 use std::path::Path;
 
 use anyhow::{bail, Context, Result};
@@ -11,13 +11,15 @@ use itertools::Itertools;
 use rustc_hash::FxHashMap;
 use tracing::{debug, instrument};
 
-use distribution_types::{CachedDist, IndexLocations, Name, Requirement, Resolution, SourceDist};
+use distribution_types::{CachedDist, IndexLocations, Name, Resolution, SourceDist};
+use pypi_types::Requirement;
 use uv_build::{SourceBuild, SourceBuildContext};
 use uv_cache::Cache;
 use uv_client::RegistryClient;
-use uv_configuration::Concurrency;
 use uv_configuration::{BuildKind, ConfigSettings, NoBinary, NoBuild, Reinstall, SetupPyStrategy};
+use uv_configuration::{Concurrency, PreviewMode};
 use uv_distribution::DistributionDatabase;
+use uv_git::GitResolver;
 use uv_installer::{Downloader, Installer, Plan, Planner, SitePackages};
 use uv_interpreter::{Interpreter, PythonEnvironment};
 use uv_resolver::{FlatIndex, InMemoryIndex, Manifest, Options, PythonRequirement, Resolver};
@@ -32,6 +34,7 @@ pub struct BuildDispatch<'a> {
     index_locations: &'a IndexLocations,
     flat_index: &'a FlatIndex,
     index: &'a InMemoryIndex,
+    git: &'a GitResolver,
     in_flight: &'a InFlight,
     setup_py: SetupPyStrategy,
     build_isolation: BuildIsolation<'a>,
@@ -43,6 +46,7 @@ pub struct BuildDispatch<'a> {
     options: Options,
     build_extra_env_vars: FxHashMap<OsString, OsString>,
     concurrency: Concurrency,
+    preview_mode: PreviewMode,
 }
 
 impl<'a> BuildDispatch<'a> {
@@ -54,6 +58,7 @@ impl<'a> BuildDispatch<'a> {
         index_locations: &'a IndexLocations,
         flat_index: &'a FlatIndex,
         index: &'a InMemoryIndex,
+        git: &'a GitResolver,
         in_flight: &'a InFlight,
         setup_py: SetupPyStrategy,
         config_settings: &'a ConfigSettings,
@@ -62,6 +67,7 @@ impl<'a> BuildDispatch<'a> {
         no_build: &'a NoBuild,
         no_binary: &'a NoBinary,
         concurrency: Concurrency,
+        preview_mode: PreviewMode,
     ) -> Self {
         Self {
             client,
@@ -70,6 +76,7 @@ impl<'a> BuildDispatch<'a> {
             index_locations,
             flat_index,
             index,
+            git,
             in_flight,
             setup_py,
             config_settings,
@@ -81,6 +88,7 @@ impl<'a> BuildDispatch<'a> {
             source_build_context: SourceBuildContext::default(),
             options: Options::default(),
             build_extra_env_vars: FxHashMap::default(),
+            preview_mode,
         }
     }
 
@@ -89,6 +97,21 @@ impl<'a> BuildDispatch<'a> {
         self.options = options;
         self
     }
+
+    /// Set the environment variables to be used when building a source distribution.
+    #[must_use]
+    pub fn with_build_extra_env_vars<I, K, V>(mut self, sdist_build_env_variables: I) -> Self
+    where
+        I: IntoIterator<Item = (K, V)>,
+        K: AsRef<OsStr>,
+        V: AsRef<OsStr>,
+    {
+        self.build_extra_env_vars = sdist_build_env_variables
+            .into_iter()
+            .map(|(key, value)| (key.as_ref().to_owned(), value.as_ref().to_owned()))
+            .collect();
+        self
+    }
 }
 
 impl<'a> BuildContext for BuildDispatch<'a> {
@@ -98,6 +121,10 @@ impl<'a> BuildContext for BuildDispatch<'a> {
         self.cache
     }
 
+    fn git(&self) -> &GitResolver {
+        self.git
+    }
+
     fn interpreter(&self) -> &Interpreter {
         self.interpreter
     }
@@ -123,9 +150,8 @@ impl<'a> BuildContext for BuildDispatch<'a> {
     }
 
     async fn resolve<'data>(&'data self, requirements: &'data [Requirement]) -> Result<Resolution> {
+        let python_requirement = PythonRequirement::from_interpreter(self.interpreter);
         let markers = self.interpreter.markers();
-        let python_requirement =
-            PythonRequirement::from_marker_environment(self.interpreter, markers);
         let tags = self.interpreter.tags()?;
         let resolver = Resolver::new(
             Manifest::simple(requirements.to_vec()),
@@ -138,7 +164,12 @@ impl<'a> BuildContext for BuildDispatch<'a> {
             &HashStrategy::None,
             self,
             EmptyInstalledPackages,
-            DistributionDatabase::new(self.client, self, self.concurrency.downloads),
+            DistributionDatabase::new(
+                self.client,
+                self,
+                self.concurrency.downloads,
+                self.preview_mode,
+            ),
         )?;
         let graph = resolver.resolve().await.with_context(|| {
             format!(
@@ -185,9 +216,9 @@ impl<'a> BuildContext for BuildDispatch<'a> {
             extraneous: _,
         } = Planner::new(&requirements).build(
             site_packages,
-            &Reinstall::None,
-            &NoBinary::None,
-            &HashStrategy::None,
+            &Reinstall::default(),
+            &NoBinary::default(),
+            &HashStrategy::default(),
             self.index_locations,
             self.cache(),
             venv,
@@ -220,7 +251,12 @@ impl<'a> BuildContext for BuildDispatch<'a> {
                 self.cache,
                 tags,
                 &HashStrategy::None,
-                DistributionDatabase::new(self.client, self, self.concurrency.downloads),
+                DistributionDatabase::new(
+                    self.client,
+                    self,
+                    self.concurrency.downloads,
+                    self.preview_mode,
+                ),
             );
 
             debug!(
