@@ -14,6 +14,7 @@ pub use crate::target::Target;
 pub use crate::virtualenv::{Error as VirtualEnvError, PyVenvConfiguration, VirtualEnvironment};
 
 mod discovery;
+pub mod downloads;
 mod environment;
 mod implementation;
 mod interpreter;
@@ -191,9 +192,9 @@ mod tests {
                             "python_version": "{VERSION}",
                             "sys_platform": "linux"
                         },
-                        "base_exec_prefix": "/home/ferris/.pyenv/versions/{FULL_VERSION}",
-                        "base_prefix": "/home/ferris/.pyenv/versions/{FULL_VERSION}",
-                        "prefix": "{PREFIX}",
+                        "sys_base_exec_prefix": "/home/ferris/.pyenv/versions/{FULL_VERSION}",
+                        "sys_base_prefix": "/home/ferris/.pyenv/versions/{FULL_VERSION}",
+                        "sys_prefix": "{PREFIX}",
                         "sys_executable": "{PATH}",
                         "sys_path": [
                             "/home/ferris/.pyenv/versions/{FULL_VERSION}/lib/python{VERSION}/lib/python{VERSION}",
@@ -1421,7 +1422,11 @@ mod tests {
             matches!(
                 result,
                 Err(Error::Discovery(
-                    crate::discovery::Error::SourceNotSelected(_, InterpreterSource::ProvidedPath)
+                    crate::discovery::Error::SourceNotSelected(
+                        _,
+                        InterpreterSource::ProvidedPath,
+                        _
+                    )
                 ))
             ),
             // TODO(zanieb): We should allow this, just enforce it's a virtualenv
