@@ -1,7 +1,8 @@
-use anyhow::Result;
+use itertools::Itertools;
 use pubgrub::range::Range;
+use std::ops::Bound;
 
-use pep440_rs::{Operator, PreRelease, Version, VersionSpecifier};
+use pep440_rs::{Operator, PreRelease, Version, VersionSpecifier, VersionSpecifiers};
 
 use crate::ResolveError;
 
@@ -9,6 +10,12 @@ use crate::ResolveError;
 #[derive(Debug)]
 pub(crate) struct PubGrubSpecifier(Range<Version>);
 
+impl PubGrubSpecifier {
+    pub(crate) fn iter(&self) -> impl Iterator<Item = (&Bound<Version>, &Bound<Version>)> {
+        self.0.iter()
+    }
+}
+
 impl From<PubGrubSpecifier> for Range<Version> {
     /// Convert a PubGrub specifier to a range of versions.
     fn from(specifier: PubGrubSpecifier) -> Self {
@@ -16,10 +23,25 @@ impl From<PubGrubSpecifier> for Range<Version> {
     }
 }
 
+impl TryFrom<&VersionSpecifiers> for PubGrubSpecifier {
+    type Error = ResolveError;
+
+    /// Convert a PEP 440 specifier to a PubGrub-compatible version range.
+    fn try_from(specifiers: &VersionSpecifiers) -> Result<Self, ResolveError> {
+        let range = specifiers
+            .iter()
+            .map(crate::pubgrub::PubGrubSpecifier::try_from)
+            .fold_ok(Range::full(), |range, specifier| {
+                range.intersection(&specifier.into())
+            })?;
+        Ok(Self(range))
+    }
+}
+
 impl TryFrom<&VersionSpecifier> for PubGrubSpecifier {
     type Error = ResolveError;
 
-    /// Convert a PEP 508 specifier to a PubGrub-compatible version range.
+    /// Convert a PEP 440 specifier to a PubGrub-compatible version range.
     fn try_from(specifier: &VersionSpecifier) -> Result<Self, ResolveError> {
         let ranges = match specifier.operator() {
             Operator::Equal => {
