@@ -56,7 +56,8 @@ impl BatchPrefetcher {
         let PubGrubPackageInner::Package {
             name,
             extra: None,
-            marker: None,
+            dev: None,
+            marker: _marker,
             url: None,
         } = &**next
         else {
