@@ -1,14 +1,13 @@
 //! Given a set of requirements, find a set of compatible packages.
 
 use std::borrow::Cow;
+use std::collections::BTreeMap;
 use std::fmt::{Display, Formatter};
-use std::ops::Deref;
 use std::sync::Arc;
 use std::thread;
 
 use dashmap::DashMap;
 use futures::{FutureExt, StreamExt, TryFutureExt};
-use itertools::Itertools;
 use pubgrub::error::PubGrubError;
 use pubgrub::range::Range;
 use pubgrub::solver::{Incompatibility, State};
@@ -20,7 +19,8 @@ use tracing::{debug, enabled, instrument, trace, warn, Level};
 
 use distribution_types::{
     BuiltDist, Dist, DistributionMetadata, IncompatibleDist, IncompatibleSource, IncompatibleWheel,
-    InstalledDist, RemoteSource, ResolvedDist, ResolvedDistRef, SourceDist, VersionOrUrlRef,
+    InstalledDist, PythonRequirementKind, RemoteSource, ResolvedDist, ResolvedDistRef, SourceDist,
+    VersionOrUrlRef,
 };
 pub(crate) use locals::Locals;
 use pep440_rs::{Version, MIN_VERSION};
@@ -31,7 +31,7 @@ pub(crate) use urls::Urls;
 use uv_configuration::{Constraints, Overrides};
 use uv_distribution::{ArchiveMetadata, DistributionDatabase};
 use uv_git::GitResolver;
-use uv_normalize::{ExtraName, PackageName};
+use uv_normalize::{ExtraName, GroupName, PackageName};
 use uv_types::{BuildContext, HashStrategy, InstalledPackagesProvider};
 
 use crate::candidate_selector::{CandidateDist, CandidateSelector};
@@ -81,6 +81,7 @@ struct ResolverState<InstalledPackages: InstalledPackagesProvider> {
     requirements: Vec<Requirement>,
     constraints: Constraints,
     overrides: Overrides,
+    dev: Vec<GroupName>,
     preferences: Preferences,
     git: GitResolver,
     exclusions: Exclusions,
@@ -191,6 +192,7 @@ impl<Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvider>
             requirements: manifest.requirements,
             constraints: manifest.constraints,
             overrides: manifest.overrides,
+            dev: manifest.dev,
             preferences: Preferences::from_iter(manifest.preferences, markers),
             exclusions: manifest.exclusions,
             hasher: hasher.clone(),
@@ -260,16 +262,12 @@ impl<Provider: ResolverProvider, InstalledPackages: InstalledPackagesProvider>
                 // Add version information to improve unsat error messages.
                 Err(if let ResolveError::NoSolution(err) = err {
                     ResolveError::NoSolution(
-                        err.with_available_versions(
-                            &state.python_requirement,
-                            &visited,
-                            state.index.packages(),
-                        )
-                        .with_selector(state.selector.clone())
-                        .with_python_requirement(&state.python_requirement)
-                        .with_index_locations(provider.index_locations())
-                        .with_unavailable_packages(&state.unavailable_packages)
-                        .with_incomplete_packages(&state.incomplete_packages),
+                        err.with_available_versions(&visited, state.index.packages())
+                            .with_selector(state.selector.clone())
+                            .with_python_requirement(&state.python_requirement)
+                            .with_index_locations(provider.index_locations())
+                            .with_unavailable_packages(&state.unavailable_packages)
+                            .with_incomplete_packages(&state.incomplete_packages),
                     )
                 } else {
                     err
@@ -311,9 +309,12 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
         let mut resolutions = vec![];
 
         debug!(
-            "Solving with target Python version {}",
-            self.python_requirement.target()
+            "Solving with installed Python version: {}",
+            self.python_requirement.installed()
         );
+        if let Some(target) = self.python_requirement.target() {
+            debug!("Solving with target Python version: {}", target);
+        }
 
         'FORK: while let Some(mut state) = forked_states.pop() {
             loop {
@@ -406,32 +407,37 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                         if let UnavailableVersion::IncompatibleDist(
                             IncompatibleDist::Source(IncompatibleSource::RequiresPython(
                                 requires_python,
+                                kind,
                             ))
                             | IncompatibleDist::Wheel(IncompatibleWheel::RequiresPython(
                                 requires_python,
+                                kind,
                             )),
                         ) = reason
                         {
-                            let python_version = requires_python
-                                .iter()
-                                .map(PubGrubSpecifier::try_from)
-                                .fold_ok(Range::full(), |range, specifier| {
-                                    range.intersection(&specifier.into())
-                                })?;
+                            let python_version: Range<Version> =
+                                PubGrubSpecifier::try_from(&requires_python)?.into();
 
                             let package = &state.next;
-                            for kind in [PubGrubPython::Installed, PubGrubPython::Target] {
-                                state.pubgrub.add_incompatibility(
-                                    Incompatibility::from_dependency(
-                                        package.clone(),
-                                        Range::singleton(version.clone()),
-                                        (
-                                            PubGrubPackage::from(PubGrubPackageInner::Python(kind)),
-                                            python_version.clone(),
-                                        ),
+                            state
+                                .pubgrub
+                                .add_incompatibility(Incompatibility::from_dependency(
+                                    package.clone(),
+                                    Range::singleton(version.clone()),
+                                    (
+                                        PubGrubPackage::from(PubGrubPackageInner::Python(
+                                            match kind {
+                                                PythonRequirementKind::Installed => {
+                                                    PubGrubPython::Installed
+                                                }
+                                                PythonRequirementKind::Target => {
+                                                    PubGrubPython::Target
+                                                }
+                                            },
+                                        )),
+                                        python_version.clone(),
                                     ),
-                                );
-                            }
+                                ));
                             state
                                 .pubgrub
                                 .partial_solution
@@ -554,7 +560,13 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
         for resolution in resolutions {
             combined.union(resolution);
         }
-        ResolutionGraph::from_state(&self.index, &self.preferences, &self.git, combined)
+        ResolutionGraph::from_state(
+            &self.index,
+            &self.preferences,
+            &self.git,
+            &self.python_requirement,
+            combined,
+        )
     }
 
     /// Visit a [`PubGrubPackage`] prior to selection. This should be called on a [`PubGrubPackage`]
@@ -568,6 +580,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
             PubGrubPackageInner::Root(_) => {}
             PubGrubPackageInner::Python(_) => {}
             PubGrubPackageInner::Extra { .. } => {}
+            PubGrubPackageInner::Dev { .. } => {}
             PubGrubPackageInner::Package {
                 name, url: None, ..
             } => {
@@ -613,6 +626,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
             let PubGrubPackageInner::Package {
                 name,
                 extra: None,
+                dev: None,
                 marker: _marker,
                 url: None,
             } = &**package
@@ -642,22 +656,10 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                 Ok(Some(ResolverVersion::Available(MIN_VERSION.clone())))
             }
 
-            PubGrubPackageInner::Python(PubGrubPython::Installed) => {
-                let version = self.python_requirement.installed();
-                if range.contains(version) {
-                    Ok(Some(ResolverVersion::Available(version.deref().clone())))
-                } else {
-                    Ok(None)
-                }
-            }
-
-            PubGrubPackageInner::Python(PubGrubPython::Target) => {
-                let version = self.python_requirement.target();
-                if range.contains(version) {
-                    Ok(Some(ResolverVersion::Available(version.deref().clone())))
-                } else {
-                    Ok(None)
-                }
+            PubGrubPackageInner::Python(_) => {
+                // Dependencies on Python are only added when a package is incompatible; as such,
+                // we don't need to do anything here.
+                Ok(None)
             }
 
             PubGrubPackageInner::Extra {
@@ -665,6 +667,11 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                 url: Some(url),
                 ..
             }
+            | PubGrubPackageInner::Dev {
+                name,
+                url: Some(url),
+                ..
+            }
             | PubGrubPackageInner::Package {
                 name,
                 url: Some(url),
@@ -722,12 +729,27 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
 
                 // The version is incompatible due to its Python requirement.
                 if let Some(requires_python) = metadata.requires_python.as_ref() {
-                    let target = self.python_requirement.target();
-                    if !requires_python.contains(target) {
+                    if let Some(target) = self.python_requirement.target() {
+                        if !target.contains(requires_python) {
+                            return Ok(Some(ResolverVersion::Unavailable(
+                                version.clone(),
+                                UnavailableVersion::IncompatibleDist(IncompatibleDist::Source(
+                                    IncompatibleSource::RequiresPython(
+                                        requires_python.clone(),
+                                        PythonRequirementKind::Target,
+                                    ),
+                                )),
+                            )));
+                        }
+                    }
+                    if !requires_python.contains(self.python_requirement.installed()) {
                         return Ok(Some(ResolverVersion::Unavailable(
                             version.clone(),
                             UnavailableVersion::IncompatibleDist(IncompatibleDist::Source(
-                                IncompatibleSource::RequiresPython(requires_python.clone()),
+                                IncompatibleSource::RequiresPython(
+                                    requires_python.clone(),
+                                    PythonRequirementKind::Installed,
+                                ),
                             )),
                         )));
                     }
@@ -739,6 +761,9 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
             PubGrubPackageInner::Extra {
                 name, url: None, ..
             }
+            | PubGrubPackageInner::Dev {
+                name, url: None, ..
+            }
             | PubGrubPackageInner::Package {
                 name, url: None, ..
             } => {
@@ -858,9 +883,13 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
             let name = match &**pkg {
                 // A root can never be a dependency of another package, and a `Python` pubgrub
                 // package is never returned by `get_dependencies`. So these cases never occur.
+                // TODO(charlie): This might be overly conservative for `Extra` and `Group`. If
+                // multiple groups are enabled, we shouldn't need to fork. Similarly, if multiple
+                // extras are enabled, we shouldn't need to fork.
                 PubGrubPackageInner::Root(_) | PubGrubPackageInner::Python(_) => unreachable!(),
                 PubGrubPackageInner::Package { ref name, .. }
-                | PubGrubPackageInner::Extra { ref name, .. } => name,
+                | PubGrubPackageInner::Extra { ref name, .. }
+                | PubGrubPackageInner::Dev { ref name, .. } => name,
             };
             by_grouping
                 .entry(name)
@@ -906,10 +935,12 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                 // Add the root requirements.
                 let dependencies = PubGrubDependencies::from_requirements(
                     &self.requirements,
+                    &BTreeMap::default(),
                     &self.constraints,
                     &self.overrides,
                     None,
                     None,
+                    None,
                     &self.urls,
                     &self.locals,
                     &self.git,
@@ -943,11 +974,27 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
             PubGrubPackageInner::Package {
                 name,
                 extra,
+                dev,
                 marker,
                 url,
             } => {
                 // If we're excluding transitive dependencies, short-circuit.
                 if self.dependency_mode.is_direct() {
+                    // If a package has a marker, add a dependency from it to the
+                    // same package without markers.
+                    if marker.is_some() {
+                        return Ok(Dependencies::Available(vec![(
+                            PubGrubPackage::from(PubGrubPackageInner::Package {
+                                name: name.clone(),
+                                extra: extra.clone(),
+                                dev: dev.clone(),
+                                marker: None,
+                                url: url.clone(),
+                            }),
+                            Range::singleton(version.clone()),
+                        )]));
+                    }
+
                     // If an extra is provided, wait for the metadata to be available, since it's
                     // still required for generating the lock file.
                     let dist = match url {
@@ -1042,18 +1089,14 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     }
                 };
 
-                let requirements: Vec<_> = metadata
-                    .requires_dist
-                    .iter()
-                    .cloned()
-                    .map(Requirement::from)
-                    .collect();
                 let mut dependencies = PubGrubDependencies::from_requirements(
-                    &requirements,
+                    &metadata.requires_dist,
+                    &metadata.dev_dependencies,
                     &self.constraints,
                     &self.overrides,
                     Some(name),
                     extra.as_ref(),
+                    dev.as_ref(),
                     &self.urls,
                     &self.locals,
                     &self.git,
@@ -1070,6 +1113,25 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     self.visit_package(dep_package, request_sink)?;
                 }
 
+                // If a package has metadata for an enabled dependency group,
+                // add a dependency from it to the same package with the group
+                // enabled.
+                if extra.is_none() && dev.is_none() {
+                    for dev in &self.dev {
+                        if metadata.dev_dependencies.contains_key(dev) {
+                            dependencies.push(
+                                PubGrubPackage::from(PubGrubPackageInner::Dev {
+                                    name: name.clone(),
+                                    dev: dev.clone(),
+                                    marker: marker.clone(),
+                                    url: url.clone(),
+                                }),
+                                Range::singleton(version.clone()),
+                            );
+                        }
+                    }
+                }
+
                 // If a package has a marker, add a dependency from it to the
                 // same package without markers.
                 //
@@ -1086,6 +1148,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                         PubGrubPackage::from(PubGrubPackageInner::Package {
                             name: name.clone(),
                             extra: extra.clone(),
+                            dev: dev.clone(),
                             marker: None,
                             url: url.clone(),
                         }),
@@ -1107,6 +1170,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     PubGrubPackage::from(PubGrubPackageInner::Package {
                         name: name.clone(),
                         extra: None,
+                        dev: None,
                         marker: marker.clone(),
                         url: url.clone(),
                     }),
@@ -1116,6 +1180,36 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     PubGrubPackage::from(PubGrubPackageInner::Package {
                         name: name.clone(),
                         extra: Some(extra.clone()),
+                        dev: None,
+                        marker: marker.clone(),
+                        url: url.clone(),
+                    }),
+                    Range::singleton(version.clone()),
+                ),
+            ])),
+
+            // Add a dependency on both the development dependency group and base package.
+            PubGrubPackageInner::Dev {
+                name,
+                dev,
+                marker,
+                url,
+            } => Ok(Dependencies::Available(vec![
+                (
+                    PubGrubPackage::from(PubGrubPackageInner::Package {
+                        name: name.clone(),
+                        extra: None,
+                        dev: None,
+                        marker: marker.clone(),
+                        url: url.clone(),
+                    }),
+                    Range::singleton(version.clone()),
+                ),
+                (
+                    PubGrubPackage::from(PubGrubPackageInner::Package {
+                        name: name.clone(),
+                        extra: None,
+                        dev: Some(dev.clone()),
                         marker: marker.clone(),
                         url: url.clone(),
                     }),
@@ -1351,6 +1445,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                 PubGrubPackageInner::Root(_) => {}
                 PubGrubPackageInner::Python(_) => {}
                 PubGrubPackageInner::Extra { .. } => {}
+                PubGrubPackageInner::Dev { .. } => {}
                 PubGrubPackageInner::Package {
                     name,
                     url: Some(url),
@@ -1442,6 +1537,7 @@ impl SolveState {
                 let PubGrubPackageInner::Package {
                     name: ref self_name,
                     extra: ref self_extra,
+                    dev: ref self_dev,
                     ..
                 } = &**self_package
                 else {
@@ -1452,6 +1548,7 @@ impl SolveState {
                     PubGrubPackageInner::Package {
                         name: ref dependency_name,
                         extra: ref dependency_extra,
+                        dev: ref dependency_dev,
                         ..
                     } => {
                         if self_name == dependency_name {
@@ -1464,8 +1561,10 @@ impl SolveState {
                         let versions = ResolutionDependencyVersions {
                             from_version: self_version.clone(),
                             from_extra: self_extra.clone(),
+                            from_dev: self_dev.clone(),
                             to_version: dependency_version.clone(),
                             to_extra: dependency_extra.clone(),
+                            to_dev: dependency_dev.clone(),
                         };
                         dependencies.entry(names).or_default().insert(versions);
                     }
@@ -1485,8 +1584,33 @@ impl SolveState {
                         let versions = ResolutionDependencyVersions {
                             from_version: self_version.clone(),
                             from_extra: self_extra.clone(),
+                            from_dev: self_dev.clone(),
                             to_version: dependency_version.clone(),
                             to_extra: Some(dependency_extra.clone()),
+                            to_dev: None,
+                        };
+                        dependencies.entry(names).or_default().insert(versions);
+                    }
+
+                    PubGrubPackageInner::Dev {
+                        name: ref dependency_name,
+                        dev: ref dependency_dev,
+                        ..
+                    } => {
+                        if self_name == dependency_name {
+                            continue;
+                        }
+                        let names = ResolutionDependencyNames {
+                            from: self_name.clone(),
+                            to: dependency_name.clone(),
+                        };
+                        let versions = ResolutionDependencyVersions {
+                            from_version: self_version.clone(),
+                            from_extra: self_extra.clone(),
+                            from_dev: self_dev.clone(),
+                            to_version: dependency_version.clone(),
+                            to_extra: None,
+                            to_dev: Some(dependency_dev.clone()),
                         };
                         dependencies.entry(names).or_default().insert(versions);
                     }
@@ -1525,8 +1649,10 @@ pub(crate) struct ResolutionDependencyNames {
 pub(crate) struct ResolutionDependencyVersions {
     pub(crate) from_version: Version,
     pub(crate) from_extra: Option<ExtraName>,
+    pub(crate) from_dev: Option<GroupName>,
     pub(crate) to_version: Version,
     pub(crate) to_extra: Option<ExtraName>,
+    pub(crate) to_dev: Option<GroupName>,
 }
 
 impl Resolution {
