@@ -215,6 +215,13 @@ pub(crate) async fn pip_compile(
         InMemoryIndexRef::Borrowed(&source_index)
     };
 
+    // Determine the Python requirement, based on the interpreter and the requested version.
+    let python_requirement = if let Some(python_version) = python_version.as_ref() {
+        PythonRequirement::from_python_version(&interpreter, python_version)
+    } else {
+        PythonRequirement::from_interpreter(&interpreter)
+    };
+
     // Determine the tags, markers, and interpreter to use for resolution.
     let tags = match (python_platform, python_version.as_ref()) {
         (Some(python_platform), Some(python_version)) => Cow::Owned(Tags::from_env(
@@ -251,8 +258,6 @@ pub(crate) async fn pip_compile(
         (None, None) => Cow::Borrowed(interpreter.markers()),
     };
 
-    let python_requirement = PythonRequirement::from_marker_environment(&interpreter, &markers);
-
     // Generate, but don't enforce hashes for the requirements.
     let hasher = if generate_hashes {
         HashStrategy::Generate
@@ -458,6 +463,9 @@ pub(crate) async fn pip_compile(
     let constraints = Constraints::from_requirements(constraints);
     let overrides = Overrides::from_requirements(overrides);
 
+    // Ignore development dependencies.
+    let dev = Vec::default();
+
     // Determine any lookahead requirements.
     let lookaheads = match dependency_mode {
         DependencyMode::Transitive => {
@@ -481,6 +489,7 @@ pub(crate) async fn pip_compile(
         requirements,
         constraints,
         overrides,
+        dev,
         preferences,
         project,
         // Do not consider any installed packages during resolution.
