@@ -256,6 +256,9 @@ pub(crate) async fn pip_sync(
     let preferences = Vec::default();
     let git = GitResolver::default();
 
+    // Ignore development dependencies.
+    let dev = Vec::default();
+
     // Create a build dispatch for resolution.
     let resolve_dispatch = BuildDispatch::new(
         &client,
@@ -292,6 +295,7 @@ pub(crate) async fn pip_sync(
         requirements,
         constraints,
         overrides,
+        dev,
         source_trees,
         project,
         &extras,
@@ -303,6 +307,7 @@ pub(crate) async fn pip_sync(
         interpreter,
         &tags,
         Some(&markers),
+        None,
         &client,
         &flat_index,
         &index,
